#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""`cssmin` - A Python port of the YUI CSS compressor.

http://pypi.python.org/pypi/cssmin/

`cssmin.py` - A Python port of the YUI CSS compressor.

Copyright (c) 2010 Zachary Voase

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

-------------------------------------------------------------------------------

This software contains portions of the YUI CSS Compressor, notably some regular
expressions for reducing the size of CSS. The YUI Compressor source code can be
found at <http://github.com/yui/yuicompressor>, and is licensed as follows:

> YUI Compressor Copyright License Agreement (BSD License)
>
> Copyright (c) 2009, Yahoo! Inc.
> All rights reserved.
>
> Redistribution and use of this software in source and binary forms,
> with or without modification, are permitted provided that the following
> conditions are met:
>
> * Redistributions of source code must retain the above
>   copyright notice, this list of conditions and the
>   following disclaimer.
>
> * Redistributions in binary form must reproduce the above
>   copyright notice, this list of conditions and the
>   following disclaimer in the documentation and/or other
>   materials provided with the distribution.
>
> * Neither the name of Yahoo! Inc. nor the names of its
>   contributors may be used to endorse or promote products
>   derived from this software without specific prior
>   written permission of Yahoo! Inc.
>
> THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
> AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
> IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
> DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
> FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
> DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
> SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
> CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
> OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
> OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""


from StringIO import StringIO # The pure-Python StringIO supports unicode.
import re


__version__ = '0.1.4'


def remove_comments(css):
    """Remove all CSS comment blocks."""

    iemac = False
    preserve = False
    comment_start = css.find("/*")
    while comment_start >= 0:
        # Preserve comments that look like `/*!...*/`.
        # Slicing is used to make sure we don"t get an IndexError.
        preserve = css[comment_start + 2:comment_start + 3] == "!"

        comment_end = css.find("*/", comment_start + 2)
        if comment_end < 0:
            if not preserve:
                css = css[:comment_start]
                break
        elif comment_end >= (comment_start + 2):
            if css[comment_end - 1] == "\\":
                # This is an IE Mac-specific comment; leave this one and the
                # following one alone.
                comment_start = comment_end + 2
                iemac = True
            elif iemac:
                comment_start = comment_end + 2
                iemac = False
            elif not preserve:
                css = css[:comment_start] + css[comment_end + 2:]
            else:
                comment_start = comment_end + 2
        comment_start = css.find("/*", comment_start)

    return css


def remove_unnecessary_whitespace(css):
    """Remove unnecessary whitespace characters."""

    def pseudoclasscolon(css):

        """
        Prevents 'p :link' from becoming 'p:link'.

        Translates 'p :link' into 'p ___PSEUDOCLASSCOLON___link'; this is
        translated back again later.
        """

        regex = re.compile(r"(^|\})(([^\{\:])+\:)+([^\{]*\{)")
        match = regex.search(css)
        while match:
            css = ''.join([
                css[:match.start()],
                match.group().replace(":", "___PSEUDOCLASSCOLON___"),
                css[match.end():]])
            match = regex.search(css)
        return css

    css = pseudoclasscolon(css)
    # Remove spaces from before things.
    css = re.sub(r"\s+([!{};:>+\(\)\],])", r"\1", css)

    # If there is a `@charset`, then only allow one, and move to the beginning.
    css = re.sub(r"^(.*)(@charset \"[^\"]*\";)", r"\2\1", css)
    css = re.sub(r"^(\s*@charset [^;]+;\s*)+", r"\1", css)

    # Put the space back in for a few cases, such as `@media screen` and
    # `(-webkit-min-device-pixel-ratio:0)`.
    css = re.sub(r"\band\(", "and (", css)

    # Put the colons back.
    css = css.replace('___PSEUDOCLASSCOLON___', ':')

    # Remove spaces from after things.
    css = re.sub(r"([!{}:;>+\(\[,])\s+", r"\1", css)

    return css


def remove_unnecessary_semicolons(css):
    """Remove unnecessary semicolons."""

    return re.sub(r";+\}", "}", css)


def remove_empty_rules(css):
    """Remove empty rules."""

    return re.sub(r"[^\}\{]+\{\}", "", css)


def normalize_rgb_colors_to_hex(css):
    """Convert `rgb(51,102,153)` to `#336699`."""

    regex = re.compile(r"rgb\s*\(\s*([0-9,\s]+)\s*\)")
    match = regex.search(css)
    while match:
        colors = map(lambda s: s.strip(), match.group(1).split(","))
        hexcolor = '#%.2x%.2x%.2x' % tuple(map(int, colors))
        css = css.replace(match.group(), hexcolor)
        match = regex.search(css)
    return css


def condense_zero_units(css):
    """Replace `0(px, em, %, etc)` with `0`."""

    return re.sub(r"([\s:])(0)(px|em|%|in|cm|mm|pc|pt|ex)", r"\1\2", css)


def condense_multidimensional_zeros(css):
    """Replace `:0 0 0 0;`, `:0 0 0;` etc. with `:0;`."""

    css = css.replace(":0 0 0 0;", ":0;")
    css = css.replace(":0 0 0;", ":0;")
    css = css.replace(":0 0;", ":0;")

    # Revert `background-position:0;` to the valid `background-position:0 0;`.
    css = css.replace("background-position:0;", "background-position:0 0;")

    return css


def condense_floating_points(css):
    """Replace `0.6` with `.6` where possible."""

    return re.sub(r"(:|\s)0+\.(\d+)", r"\1.\2", css)


def condense_hex_colors(css):
    """Shorten colors from #AABBCC to #ABC where possible."""

    regex = re.compile(r"([^\"'=\s])(\s*)#([0-9a-fA-F])([0-9a-fA-F])([0-9a-fA-F])([0-9a-fA-F])([0-9a-fA-F])([0-9a-fA-F])")
    match = regex.search(css)
    while match:
        first = match.group(3) + match.group(5) + match.group(7)
        second = match.group(4) + match.group(6) + match.group(8)
        if first.lower() == second.lower():
            css = css.replace(match.group(), match.group(1) + match.group(2) + '#' + first)
            match = regex.search(css, match.end() - 3)
        else:
            match = regex.search(css, match.end())
    return css


def condense_whitespace(css):
    """Condense multiple adjacent whitespace characters into one."""

    return re.sub(r"\s+", " ", css)


def condense_semicolons(css):
    """Condense multiple adjacent semicolon characters into one."""

    return re.sub(r";;+", ";", css)


def wrap_css_lines(css, line_length):
    """Wrap the lines of the given CSS to an approximate length."""

    lines = []
    line_start = 0
    for i, char in enumerate(css):
        # It's safe to break after `}` characters.
        if char == '}' and (i - line_start >= line_length):
            lines.append(css[line_start:i + 1])
            line_start = i + 1

    if line_start < len(css):
        lines.append(css[line_start:])
    return '\n'.join(lines)


def cssmin(css, wrap=None):
    css = remove_comments(css)
    css = condense_whitespace(css)
    # A pseudo class for the Box Model Hack
    # (see http://tantek.com/CSS/Examples/boxmodelhack.html)
    css = css.replace('"\\"}\\""', "___PSEUDOCLASSBMH___")
    css = remove_unnecessary_whitespace(css)
    css = remove_unnecessary_semicolons(css)
    css = condense_zero_units(css)
    css = condense_multidimensional_zeros(css)
    css = condense_floating_points(css)
    css = normalize_rgb_colors_to_hex(css)
    css = condense_hex_colors(css)
    if wrap is not None:
        css = wrap_css_lines(css, wrap)
    css = css.replace("___PSEUDOCLASSBMH___", '"\\"}\\""')
    css = condense_semicolons(css)
    return css.strip()


def main():
    import optparse
    import sys

    p = optparse.OptionParser(
        prog="cssmin", version=__version__,
        usage="%prog [--wrap N]",
        description="""Reads raw CSS from stdin, and writes compressed CSS to stdout.""")

    p.add_option(
        '-w', '--wrap', type='int', default=None, metavar='N',
        help="Wrap output to approximately N chars per line.")

    options, args = p.parse_args()
    sys.stdout.write(cssmin(sys.stdin.read(), wrap=options.wrap))


if __name__ == '__main__':
    main()



"""Copyright (c) 2012 Charles Kaminski (CharlesKaminski@gmail.com)

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE."""

from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from os import getcwd, sep, walk, makedirs
from os.path import join, exists, basename, dirname
from shutil import move, rmtree
from optparse import make_option
from _TemplateTextMinimizer import minimize_template_text

ARCHIVE = '_minimizer_archive'
REVERTED = '_reverted_'
ARCHIVE_EXISTS = ('A minimizer archive folder already exists.\n'
                  'Check that the following templates are not already '
                  'minimized: \n%s')
ARCHIVE_DOESNT_EXIST = ("The below archive folder doesn't exist.\n"
                        "Check that you shouldn't be reverting the "
                        "coresponding folder: \n%s")

class Command(BaseCommand):
    help = '''Use this tool to minimize Django templates after
development.  This way, your templates are small when they are
evaluated and the HTML served is already minimized; eliminiating
any post-processing minimization step.  The command can be undone
using the -u option.

Use the comment tags {# NOMINIFY #} {# ENDNOMINIFY #} to wrap
content you do not want minified.

Uses the setting TEMPLATE_DIRS in your Django settings file to
    tell the command where to find templates to minimize.

Customization:
The minimizer command uses its own minimizers for html, style tag embeded
    css, and script tag embeded javascript. You can override these and chain
    any number of your own minimizers using settings below.  These
    settings go in the Django settings file. Custom minimizers must
    be functions that accept text as a string parameter and return
    text as a string.
    JAVASCRIPT_MINIMIZERS = [custom_function1, custom_function2,]
    CSS_MINIMIZERS        = [custom_function3, custom_function4,]
    HTML_MINIMIZERS       = [custom_function5, custom_function6,]

    To turn off a minimizer, use the following pattern:
    f = lambda x: x
    JAVASCRIPT_MINIMIZER = [f,]

You can tell the minimizer command to disable an aggressive minimizer 
    in the default HTML minimizer chain.  This minimizer normally removes 
    (instead of just collapsing) the remaining space between '>' & '<' 
    character.  Set the following setting to False in your Django
    settings file to disable this final step:
    AGGRESSIVE_HTML_MINIMIZER = False

Method - For each template, the minimizer command:
1. Replaces any {# NOMINIFY #} {# ENDNOMINIFY #} content with
    a unique identifier and saves the content in memory so that
    it is excluded from the rest of the process.
2. Remaining Django comments are removed.
3. Django tags and django variables are replaced with with unique
    identifiers.  The tags and variables are saved in memory.
    This approach "protects" the tags and variables from the
    minimizers.  It also allows you to use Django tags and variables
    inside your javascript and CSS without ill effect by the
    CSS or javascript minimizer.
4. HTML script tags and content are replaced with unique
    identifiers. The tags and content are saved in memory for
    additional processing.  The type attribute for the script tag
    is checked to see if the script content is javascript.  If no
    type is provided, then javascript is assumed.  Any javascript
    is then run through the javascript minimizers.
5. An almost identical process to step 4 is implemented on the HTML
    style tags for css.
6. The remaining text (with the identifiers) is run through the
    html minimizers.
7. All of the content saved in memory and associated with unique
    identifiers are put back.
8. The original template is moved to an archive folder and replaced
    with the minimized template.

Limitations:
The minimizer does not handle script tags inside script tags or
    style tags inside style tags; an unusual occurance.
    eg: <script>bla bla <script> bla</script></script>
The minimizer collapses all white space not in a django tag,
    django variable, javascript, or inline css.  This includes
    whitespace inside <pre>, <textarea>, & similar tags, and
    whitespace inside html attributes.
Use the {# NOMINIFY #} {# ENDNOMINIFY #} comment tags to overcome
    these limiations.
    
'''

    option_list = BaseCommand.option_list + (
        make_option('-m', '--minimize',
                    action='store_true', dest='minimize', default=False,
                    help='Minimize templates.'),
        make_option('-u', '--undo',
                    action='store_true', dest='undo', default=False,
                    help='Reverts minimized templates from the archive.'),)

    def handle(self, *args, **options):

        # Reading template location from settings
        try:
            dirs = settings.TEMPLATE_DIRS
        except AttributeError:
            raise CommandError('You must specify TEMPLATE_DIRS in your '
                               'settings file.')

        # Get the full and proper paths
        cwd = getcwd()
        dirs = [x.replace('/', sep).rstrip(sep) for x in dirs]
        dirs = [join(cwd,x) for x in dirs]

        if options['undo']:
            self.revert(dirs)
            self.stdout.write('Successfully reverted minimized Templates:\n')
            for d in dirs:
                self.stdout.write('%s\n' % d)
            return 0
        elif options['minimize']:
            self.minimize_templates(dirs)
            self.stdout.write('Successfully minimized Templates:\n')
            for d in dirs:
                self.stdout.write('%s\n' % d)
            return 0
        else:
            command_name = basename(__file__).rstrip('.py')
            self.print_help(command_name, '')

    def minimize_templates(self, dirs):

        # Check that the archive folders don't already exist
        for d in dirs:
            if exists(join(d, ARCHIVE)):
                raise CommandError(ARCHIVE_EXISTS % d)

        paths = []
        # Walk the directories to build a list of files to minimize
        # Currently not following symbolic links
        for d in dirs:
            archive_dir = join(d, ARCHIVE)
            for root, walk_dirs, files in walk(d):
                # Didn't used "startswith" on the if as am getting strange
                # behavior in windows
                walk_dirs = [x for x in walk_dirs if not REVERTED in x]
                for name in files:
                    if not name.split('.')[-1] in ('.py', '.pyc'):
                        path= join(root,name)
                        paths.append([path, path.replace(d, archive_dir)])

        # Minimize the files
        num_files, before, after = 0, 0, 0
        for source_path, archive_path in paths:
            original = ''.join(open(source_path).readlines())
            minimized = minimize_template_text(original)

            num_files = num_files + 1
            before = before + len(original)
            after = after + len(minimized)

            d = dirname(archive_path)
            if not exists(d):
                makedirs(d)
            move(source_path, archive_path)
            open(source_path, 'wb').write(minimized)

        print 'Files:    %s' % num_files
        print 'Before:   %s' % before
        print 'After:    %s' % after
        print "Decrease: {0:.0%}".format((before - after) / float(before))

    def revert(self, dirs):
        # Put together Archive dirs
        # Check  the archive folders do exist
        archive_dirs = []
        for d in dirs:
            archive_dir = join(d, ARCHIVE)
            archive_dirs.append(archive_dir)
            if not exists(archive_dir):
                raise CommandError(ARCHIVE_DOESNT_EXIST % archive_dir)

        # Put together reverted dirs
        reverted_dirs = []
        for d in dirs:
            i = 1
            while exists(join(d, REVERTED + str(i))):
                i = i + 1
            reverted_dirs.append(join(d, REVERTED + str(i)))

        # zip everything up
        all_dirs = zip(archive_dirs, dirs, reverted_dirs)

        # Walk the directories and put together paths
        paths = []
        for archive_dir, d, reverted_dir in all_dirs:
            for root, walk_dirs, files in walk(archive_dir):
                walk_dirs = [x for x in walk_dirs if not x.startswith(REVERTED)]
                for archive_name in files:
                    # Collect all the paths
                    archive_path = join(root, archive_name)
                    path = archive_path.replace(archive_dir, d)
                    reverted_path = archive_path.replace(archive_dir,
                                                         reverted_dir)
                    paths.append([archive_path, path, reverted_path])

        # Make the moves
        for archive_path, path, reverted_path in paths:
            # Create the directories
            d = dirname(path)
            if not exists(d): makedirs(d)
            d = dirname(reverted_path)
            if not exists(d): makedirs(d)

            move(path, reverted_path)
            move(archive_path, path)

        # Delete the archive folder
        for d in archive_dirs:
            rmtree(d)



"""Copyright (c) 2012 Charles Kaminski (CharlesKaminski@gmail.com)

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE."""

import re
from _SimpleHTMLParser import get_first_tag_info
from _ManageMinimizers import get_minimizers

FLAGS  = re.IGNORECASE + re.DOTALL

EXCLUDE = r'({#\s*NOMINIFY\s*#})(.*?)({#\s*ENDNOMINIFY\s*#})'
REMOVE  = r'({%\s*COMMENT\s*%})(.*?)({%\s*ENDCOMMENT\s*%})' # Tag Comments
REMOVE2 = r'({#[^\n\r]+#})' # Other Comments
DVAR    = r'({{[^\n\r]+}})' # Django variable
DTAG    = r'({%[^\n\r]+%})' # Django tag
SCRIPT  = r'(<SCRIPT\b[^>]*>)(.*?)(</SCRIPT>)'
STYLE   = r'(<STYLE\b[^>]*>)(.*?)(</STYLE>)'

RE_EXCLUDE = re.compile(EXCLUDE, flags=FLAGS)
RE_REMOVE  = re.compile(REMOVE,  flags=FLAGS)
RE_REMOVE2 = re.compile(REMOVE2, flags=FLAGS)
RE_DVAR    = re.compile(DVAR,    flags=FLAGS)
RE_DTAG    = re.compile(DTAG,    flags=FLAGS)
RE_SCRIPT  = re.compile(SCRIPT,  flags=FLAGS)
RE_STYLE   = re.compile(STYLE,   flags=FLAGS)

# Will encasing the key in quotes. Will work well with javascript minifiers?
KEY    = '_~%s~_'

JSMINIMIZERS, CSSMINIMIZERS, HTMLMINIMIZERS = get_minimizers()

def minimize_template_text(text):
    """Takes a Django template text and returns a minified version.

    Performance is not critical as this function should be run off-line
    to store minimized templates.

    This function does not properly handle script or style tags embeded inside
    themselves; an unusual occurence.

    EG:
    <script>bla
       <script>bla</script>
    </script>

    Use the {# NOMINIFY #} {# ENDNOMINIFY #} comment tags to wrap code you do
    not want minified.  Suggested uses include wrapping pre, code, and textarea
    html tags as well as the example above.
    """

    # Create a list to hold special values temporarily removed
    # from the text.
    # format -> [(key, value), ...]
    word_list = []

    # Remove Excluded text
    excludes = RE_EXCLUDE.findall(text)

    # Strip out the {# NOMINIFY #} and {# ENDNOMINIFY #} tags
    excludes_find    = [''.join(x) for x in excludes]
    excludes_replace = [x[1] for x in excludes]

    # replace excluded text with keys and populate word_list
    text = subsitute_text(text, word_list, KEY, excludes_find, excludes_replace)

    # Remove any remaining template comments
    text = RE_REMOVE.sub('', text)
    text = RE_REMOVE2.sub('', text)

    # We want to "protect" Django Vars and Tags
    # then treat scripts and styles after "protecting" Django Vars and tags

    # Extract Django Variables
    dvars = RE_DVAR.findall(text)

    # replace Django Variables text with keys and populate word_list
    text = subsitute_text(text, word_list, KEY, dvars)

    # Extract django tags
    dtags = RE_DTAG.findall(text)

    # replace Django Tags text with keys and populate word_list
    text = subsitute_text(text, word_list, KEY, dtags)

    # Extract Styles
    styles  = RE_STYLE.findall(text)

    # Minimize syles
    styles_replace = minimize_tag_data(styles) or None

    # Put style pieces together
    styles_find = [''.join(x) for x in styles]

    # replace styles with keys and populate word_list
    text = subsitute_text(text, word_list, KEY, styles_find, styles_replace)

    # Extract Scripts
    scripts = RE_SCRIPT.findall(text)

    # Put scripts pieces together
    scripts_find = [''.join(x) for x in scripts]

    # Minimize javascripts if requested
    scripts_replace = minimize_tag_data(scripts) or None

    # replace scripts with keys and populate word_list
    text = subsitute_text(text, word_list, KEY, scripts_find, scripts_replace)

    # Run HTML Minimizers
    for f in HTMLMINIMIZERS: text = f(text)

    # put values back into text
    text = revert_text_keys(text, word_list)

    return text.strip()

def revert_text_keys(text, word_list):
    """ revert the text keys.  We may have to do this multiple times
    as some keys may be embeded in other word_list entries"""

    # If we loop through the word_list 10 times without
    #   consuming it completely, then something's probably wrong.

    max_iterations = len(word_list) * 10

    for x in xrange(max_iterations):

        # When the wordlist empties, break the for loop
        if not word_list:
            break

        # Take the first entry off the list
        key, value = word_list.pop(0)

        if key in text:
            # Replace the key with the value
            text = text.replace(key, value)
        else:
            # Put the entry back at the end of the list
            word_list.append((key, value))
    else:
        # We looped through too many time and
        # never consumed the whole word_list
        raise Exception('Minimizer failed to find all embeded variables.\n'
                        '%s\n%s' % (word_list, text))

    return text

def subsitute_text(text, word_list, key_template,
                   find_list, substitute_list=None):
    """ Using the text provided, subsitutes the first occurance of each entry
        in find_list with a key.  Store the key and value in the substitute_list
        in the word_list.  If no substitute_list is provided, use the value in
        the find list"""
    if not substitute_list: substitute_list = find_list

    values_list = zip(find_list, substitute_list)

    for find, substitute in values_list:
        key = key_template % len(word_list)
        text = text.replace(find, key, 1)
        word_list.append((key, substitute))

    return text

def minimize_tag_data(tags):
    """ This minimizes data in certain tags.
    Currently <script>Javascript</script> and <style>css</style>.
    Function assumes script is javascript if no type is specified.
    Function also assumes style is css if no type is specified.
    """

    retval = []

    for tag, data, end_tag in tags:

        tag_name, attr = get_first_tag_info(tag)
        type_attr = attr.get('type', 'javascript')

        if tag_name == 'script':
            type_attr = attr.get('type', 'javascript')
            if 'javascript' in type_attr:
                for f in JSMINIMIZERS: data = f(data)

        if tag_name == 'style':
            type_attr = attr.get('type', 'css')
            if 'css' in type_attr:
                for f in CSSMINIMIZERS: data = f(data)

        retval.append(tag + data + end_tag)

    return retval

if __name__ == '__main__':
    text = """{% block one %}
{% comment %}
This should be stripped out
{% endcomment %}
{# This comment should be gone as well #}
<Style> bla bla bla </stylE>
<style type="text/css">
h1 {letter-spacing:2px;}
h2 {letter-spacing:-3px;}
p {text-indent:50px;}
</style>
<scrIPT type="Text/javascript">a</Script>
<script language=Javascript>
<!--
var _P="http://www.js-examples.com/data/ex_1004/";

function preloadImage(fnImg)
{
    // create array for preloadedImages if not already created:
    if (!document.preloadedImages)
        document.preloadedImages = new Array();
    // create new image:
    document.preloadedImages[fnImg] = new Image();
    document.preloadedImages[fnImg].src = _P+fnImg;
}

function imgSwitch(imgId, fnImgNew)
{
    // switch to new image:
    if (document.preloadedImages && document.preloadedImages[fnImgNew])
        document.images[imgId].src = document.preloadedImages[fnImgNew].src;
    else
        document.images[imgId].src = _P+fnImgNew;
}

function imgMouseOver(imgId, fnImgOver)
{
    // remember original image:
    document.images[imgId].originalSrc = document.images[imgId].src;
    // switch to mouseover image:
    imgSwitch(imgId, fnImgOver);
}

function imgMouseOut(imgId)
{
    // switch back to original image:
    if (document.images[imgId] && document.images[imgId].originalSrc)
        document.images[imgId].src = document[imgId].originalSrc;
}


var timerID, now;

// These values are to store currently-displayed date digits.
// They're initialized to -1 to ensure that all digits are updated in
//   the first call to showTime:
var year10 = -1;
var year1 = -1;
var month10 = -1;
var month1 = -1;
var date10 = -1;
var date1 = -1;
var day = -1;
var hours10 = -1;
var hours1 = -1;
var minutes10 = -1;
var minutes1 = -1;
var seconds10 = -1;
var seconds1 = -1;

// updates clock display:
function showTime()
{
    // get digits in date:
    var now = new Date();
    var newYear = now.getYear();
    newYear = newYear % 100;
    var newYear10 = Math.floor(newYear / 10);
    var newYear1 = newYear - (newYear10 * 10);
    var newMonth = now.getMonth();
    newMonth++;
    var newMonth10 = Math.floor(newMonth / 10);
    var newMonth1 = newMonth - (newMonth10 * 10);
    var newDate = now.getDate();
    var newDate10 = Math.floor(newDate / 10);
    var newDate1 = newDate - (newDate10 * 10);
    var newDay = now.getDay();
    var newHours = now.getHours();
    var newHours10 = Math.floor(newHours / 10);
    var newHours1 = newHours - (newHours10 * 10);
    var newMinutes = now.getMinutes();
    var newMinutes10 = Math.floor(newMinutes / 10);
    var newMinutes1 = newMinutes - (newMinutes10 * 10);
    var newSeconds = now.getSeconds();
    var newSeconds10 = Math.floor(newSeconds / 10);
    var newSeconds1 = newSeconds - (newSeconds10 * 10);

    // only switch image if date digit has changed:
    if (newYear10 != year10)
    {
        imgSwitch("year10", "images/tinydigit" + newYear10 + ".gif");
        year10 = newYear10;
    }
    if (newYear1 != year1)
    {
        imgSwitch("year1", "images/tinydigit" + newYear1 + ".gif");
        year1 = newYear1;
    }
    if (newMonth10 != month10)
    {
        if (newMonth10 == 0)
            imgSwitch("month10", "images/tinyblank.gif");
        else
            imgSwitch("month10", "images/tinydigit" + newMonth10 + ".gif");
        month10 = newMonth10;
    }
    if (newMonth1 != month1)
    {
        imgSwitch("month1", "images/tinydigit" + newMonth1 + ".gif");
        month1 = newMonth1;
    }
    if (newDate10 != date10)
    {
        imgSwitch("date10", "images/tinydigit" + newDate10 + ".gif");
        date10 = newDate10;
    }
    if (newDate1 != date1)
    {
        imgSwitch("date1", "images/tinydigit" + newDate1 + ".gif");
        date1 = newDate1;
    }
    if (newDay != day)
    {
        imgSwitch("day", "images/day" + newDay + ".gif");
        day = newDay;
    }
    if (newHours10 != hours10)
    {
        imgSwitch("hours10", "images/digit" + newHours10 + ".gif");
        hours10 = newHours10;
    }
    if (newHours1 != hours1)
    {
        imgSwitch("hours1", "images/digit" + newHours1 + ".gif");
        hours1 = newHours1;
    }
    if (newMinutes10 != minutes10)
    {
        imgSwitch("minutes10", "images/digit" + newMinutes10 + ".gif");
        minutes10 = newMinutes10;
    }
    if (newMinutes1 != minutes1)
    {
        imgSwitch("minutes1", "images/digit" + newMinutes1 + ".gif");
        minutes1 = newMinutes1;
    }
    if (newSeconds10 != seconds10)
    {
        imgSwitch("seconds10", "images/smalldigit" + newSeconds10 + ".gif");
        seconds10 = newSeconds10;
    }
    imgSwitch("seconds1", "images/smalldigit" + newSeconds1 + ".gif");

    // update clock again in 1 second:
    timerID = setTimeout("showTime()", 1000);
}


function preloadImages()
{
    for (var i = 0; i <= 9; i++)
    {
        preloadImage("images/digit" + i + ".gif");
        preloadImage("images/smalldigit" + i + ".gif");
        preloadImage("images/tinydigit" + i + ".gif");
    }
    for (i = 0; i <= 6; i++)
        preloadImage("images/day" + i + ".gif");
}

// -->
</script>
here \t\t\tcomes        the rain \n\n\nagain <a href='bla'>test</a>
{% block two %}
{{ fun.times|here:"bla" }}
<div>over the rainbow</div>
{#NOMINiFY #} do     not minify     this   . {# ENDNOminify

#}
"""

    a = minimize_template_text(text)
    print text
    print '#' * 10
    print a
    print '#' * 10
    print 'Character length before: %s' % len(text)
    print 'Character length after:  %s' % len(a)
